/*********************************************************************************************************************

                                     Control StreetLights using a zigbee mesh netwok
     _____ __________.___              _________ __                         __  .____    .__       .__     __
    /  _  \\______   \   |            /   _____//  |________   ____   _____/  |_|    |   |__| ____ |  |___/  |_  ______
   /  /_\  \|     ___/   |   ______   \_____  \\   __\_  __ \_/ __ \_/ __ \   __\    |   |  |/ ___\|  |  \   __\/  ___/
  /    |    \    |   |   |  /_____/   /        \|  |  |  | \/\  ___/\  ___/|  | |    |___|  / /_/  >   Y  \  |  \___ \
  \____|__  /____|   |___|           /_______  /|__|  |__|    \___  >\___  >__| |_______ \__\___  /|___|  /__| /____  >
         \/                                 \/                   \/     \/             \/ /_____/      \/          \/



    Auteur  : MS
    Date    : 26/06/2018
    Version : 2.0
    Revision: V0.2 20/06/2018 Manual control relay and dimmer
              V0.3 21/06/2018 Manual control Setup RTC
              V0.4 22/06/2018 Compute sunset an sunrise times
              V0.5 22/06/2018 Automatic controle
              V0.6 23/06/2018 Compute summer time
              V1.0 24/06/2018 Dimming before sunset and sunrise
              V1.1 25/06/2018 Add control Mode (AUTO/MAN) / control Max light and Offset time before sunset and sunrise
              V2.0 26.06/2018 Optimisation memory code

    Organisation : Lycee Alphonse Benoit - L'Isle sur la Sorgue - 84800 - FRANCE
    Client : API Applications Pour l'Industrie - Vedene - 84270 - FRANCE

    Connections : go to Globale definitions to modify
    ------------
     - MOS canal N is connected to pin PWM 6 to conform 0-10V control
     - RTC DS3231 is connected to pin A5 and A4 on the I2C bus
     - Relay is connected to pin 13
     - Xbee 2C module is connected on software Serial Port (10,11)
          bottom view
           --------
          /        \
         /          \
        |*          *| --> 5V/3.3V
        |*          *| --> RX (10)
        |*          *| --> TX (11) (need 3.3V adaptation)
        |*          *|
        |*          *|
        |*          *|
        |*          *|
        |*          *|
        |*          *|
        |*          *| --> GND
         ------------
    Libraries :
    -----------
     - SoftwareSerial.h : to use digital pin like a serial port
     - Wire.h : to use I2C bus
     - TimeLord.h : to find compute sunset and sunrise time
     - DS3231.h : to use RTC and classes RTClib and DateTime
       WARNING : Class DateTime as a dayOfTheWeek() method declared but not instancied
                 It is necessary to compute summer and winter time days changes to adjust
                 UTC when computing sunrise and sunset times

                 Add the code below in DS3231.cpp file

                 uint8_t DateTime::dayOfTheWeek() const {
                    uint16_t day = date2days(yOff, m, d);
                    return (day + 6) % 7; // Jan 1, 2000 is a Saturday, i.e. returns 6
                }

    Description :
    -------------
    - Manual control    : according the protocol orders
    - Automatic control : according sunset and sunrise time

    Protocol :
    ----------
     - orders   : <slaveAddress><sep><order><sep><data><sep><checksum>\n
     - response : <slaveAddress><sep><order><sep><data><sep><timestamp><checksum>\n

    Separator : go to Globale definitions section to modify
    -----------
     - <sep> : ";" semicolon by default

    Oders :
    -------
     - REL : relay control, data is ON or OFF
     - DIM : dimmer control, data is a number beetwen 0 and 100
     - AMP : Energy consumption, data is 0 for a request
                                 data is a real number that is measure of energy consumption in ampere
     - RTC : RTC setup, data is a timestamp number
     - MOD : MANUAL  : Inhibit automatic mode Only manual commands are effective.
             AUTO    : Automatic mode. Manual commands are effective temporarly.
     - MAX : Setup the max % light in automatic mode
     - OFS : Setup the offset time before sunrise and sunset
     - LON : ACTUALY INHITBIT. Setup the Longitude. Default : -5.05092
     - LAT : ACTUALY INHITBIT. Setup the Latitude.  Default : 43.91915
     - EST : Enable European Summer Time (True/False)

    slaveAddress : go to Globale definitions section to modify
    --------------
     - unicast  : number 1 to 1000
     - broadcast : 0

    links to understand :
    ---------------------
     - Compute a checksum :
        https://www.scadacore.com/tools/programming-calculators/online-checksum-calculator/
     - Compute a timestamp and find date-time
        https://www.epochconverter.com/
     - Compute Sunrise and Sunset time
        https://keisan.casio.com/exec/system/1224686065

********************************************************************************************************************/
/*******************************************************************************************************************
         ____ _       _           _            _       __ _       _ _   _
        / ___| | ___ | |__   __ _| | ___    __| | ___ / _(_)_ __ (_) |_(_) ___  _ __  ___
       | |  _| |/ _ \| '_ \ / _` | |/ _ \  / _` |/ _ \ |_| | '_ \| | __| |/ _ \| '_ \/ __|
       | |_| | | (_) | |_) | (_| | |  __/ | (_| |  __/  _| | | | | | |_| | (_) | | | \__ \
        \____|_|\___/|_.__/ \__,_|_|\___|  \__,_|\___|_| |_|_| |_|_|\__|_|\___/|_| |_|___/

 *******************************************************************************************************************/
#define RELAY_PIN 13
#define DIMMER_PIN 6
#define CURRENT_PIN A0

#define diffusionAddress "0"
#define slaveAddress "1"

#define SEPARATOR ';'
#define END_OF_FRAME '\n'
#define ON HIGH
#define OFF LOW

/*******************************************************************************************************************

        ____       _                       _       __ _       _ _   _
       |  _ \  ___| |__  _   _  __ _    __| | ___ / _(_)_ __ (_) |_(_) ___  _ __  ___
       | | | |/ _ \ '_ \| | | |/ _` |  / _` |/ _ \ |_| | '_ \| | __| |/ _ \| '_ \/ __|
       | |_| |  __/ |_) | |_| | (_| | | (_| |  __/  _| | | | | | |_| | (_) | | | \__ \
       |____/ \___|_.__/ \__,_|\__, |  \__,_|\___|_| |_|_| |_|_|\__|_|\___/|_| |_|___/
                               |___/

       Comment it if not needed
 ******************************************************************************************************************/
//#define DEBUG_CHECKSUM
//#define DEBUG_TEST_CHECKSUM
#define DEBUG_SHOW_FRAME
#define DEBUG_DIMMER
#define DEBUG_AMPERE
#define DEBUG_DATETIME
#define DEBUG_SUNSET
#define DEBUG_SUNRISE
#define DEBUG_DO_AT_SUNSET
#define DEBUG_DO_AT_SUNRISE
#define DEBUG_OFS
#define DEBUG_MODE
#define DEBUG_MAX
//#define DEBUG_LATLONG
#define DEBUG_SUMMERTIME

/*******************************************************************************************************************
        _     _ _                    _
       | |   (_) |__  _ __ __ _ _ __(_) ___  ___
       | |   | | '_ \| '__/ _` | '__| |/ _ \/ __|
       | |___| | |_) | | | (_| | |  | |  __/\__ \
       |_____|_|_.__/|_|  \__,_|_|  |_|\___||___/

 ******************************************************************************************************************/
#include <SoftwareSerial.h>
#include <DS3231.h>
#include <Wire.h>
#include <TimeLord.h>

/*******************************************************************************************************************
         ____            _              _
        / ___|___  _ __ | |_ __ _ _ __ | |_ ___  ___   _______  _ __   ___  ___
       | |   / _ \| '_ \| __/ _` | '_ \| __/ _ \/ __| |_  / _ \| '_ \ / _ \/ __|
       | |__| (_) | | | | || (_| | | | | ||  __/\__ \  / / (_) | | | |  __/\__ \
        \____\___/|_| |_|\__\__,_|_| |_|\__\___||___/ /___\___/|_| |_|\___||___/

 ******************************************************************************************************************/
// For L'Isle sur la Sorgue (FR)
float const LONGITUDE = -5.05092;
float const LATITUDE = 43.91915;

/*******************************************************************************************************************
       __     __         _       _     _
       \ \   / /_ _ _ __(_) __ _| |__ | | ___  ___   _______  _ __   ___  ___
        \ \ / / _` | '__| |/ _` | '_ \| |/ _ \/ __| |_  / _ \| '_ \ / _ \/ __|
         \ V / (_| | |  | | (_| | |_) | |  __/\__ \  / / (_) | | | |  __/\__ \
          \_/ \__,_|_|  |_|\__,_|_.__/|_|\___||___/ /___\___/|_| |_|\___||___/

 ******************************************************************************************************************/

int sunset = 480; //8h default
int sunrise = 1140; //19h default

uint8_t soft = 0;
uint8_t offset = 30; //Before sunrise or sunset
uint8_t maxLight = 100; //100% when lighted

//int secondSep, thirdSep;

bool toSend = false;
bool doneMidnigth = false;
bool doneSunrise = false;
bool doneSunset = false;
bool manualMode = false;
bool EST = true; // enable Summertime

RTClib RTC;

byte today[] = {0, 0, 12, 22, 06, 18};
TimeLord tardis;

SoftwareSerial xbee(10, 11);

/*******************************************************************************************************************
        ____       _
       / ___|  ___| |_ _   _ _ __    _______  _ __   ___
       \___ \ / _ \ __| | | | '_ \  |_  / _ \| '_ \ / _ \
        ___) |  __/ |_| |_| | |_) |  / / (_) | | | |  __/
       |____/ \___|\__|\__,_| .__/  /___\___/|_| |_|\___|
                            |_|
 ******************************************************************************************************************/
void setup() {
  Serial.begin(9600);
  xbee.begin(9600);
  Wire.begin();
  pinMode(RELAY_PIN, OUTPUT);
  pinMode(DIMMER_PIN, OUTPUT);
  pinMode(CURRENT_PIN, INPUT);
  dimmer("0");
  tardis.Position(LATITUDE, LONGITUDE);
}

/*******************************************************************************************************************
        _
       | | ___   ___  _ __    _______  _ __   ___
       | |/ _ \ / _ \| '_ \  |_  / _ \| '_ \ / _ \
       | | (_) | (_) | |_) |  / / (_) | | | |  __/
       |_|\___/ \___/| .__/  /___\___/|_| |_|\___|
                     |_|
 ******************************************************************************************************************/
void loop()
{
  //  // For L'Isle sur la Sorgue (FR)
  //  float LONGITUDE = -5.05092;
  //  float LATITUDE = 43.91915;
  //  RTClib RTC;
  DateTime now = RTC.now();
  uint32_t tsNow = now.unixtime();

  if (!manualMode)
  {
    ////////////////////////////////////////////////////////////////////////////////
    //   _______                               _     _       _       _
    //  (_______)                             (_)   | |     (_)     | |    _
    //   _____ _   _ ____  ____ _   _    ____  _  _ | |____  _  ____| | _ | |_
    //  |  ___) | | / _  )/ ___) | | |  |    \| |/ || |  _ \| |/ _  | || \|  _)
    //  | |____\ V ( (/ /| |   | |_| |  | | | | ( (_| | | | | ( ( | | | | | |__
    //  |_______)_/ \____)_|    \__  |  |_|_|_|_|\____|_| |_|_|\_|| |_| |_|\___)
    //                         (____/                         (_____|
    //
    //                     refresh sunrise and sunset hour
    ////////////////////////////////////////////////////////////////////////////////
    if (midnight(tsNow))
    {
      if (!doneMidnigth)
      {
        doneMidnigth = true;
        computeSunriseSunset(now);
      }
    }
    else doneMidnigth = false;


    //////////////////////////////////////////////////////////////////
    //      _                      _                 _____     _
    //     | |                    (_)               (_____)   | |
    //      \ \  _   _ ____   ____ _  ___  ____        _  ___ | | _
    //       \ \| | | |  _ \ / ___) |/___)/ _  )      | |/ _ \| || \ 
    //   _____) ) |_| | | | | |   | |___ ( (/ /    ___| | |_| | |_) )
    //  (______/ \____|_| |_|_|   |_(___/ \____)  (____/ \___/|____/
    //
    //////////////////////////////////////////////////////////////////
    int minutes = getMinutesOnDay(now);
    int beforeSunrise = offset - soft;
    if (minutes == (sunrise - beforeSunrise))
    {
#ifdef DEBUG_DO_AT_SUNRISE
      Serial.print(minutes);
      Serial.print(" -> Sunrise : ");
      Serial.print(sunrise);
      Serial.print(" -> before sunrise : ");
      Serial.println(beforeSunrise);
#endif
      if (minutes <= sunrise)
      {
        uint8_t d = maxLight * ((float)(offset - soft) / offset);
        if (d < 5) d = 0;
        String dim = (String)d;

#ifdef DEBUG_DO_AT_SUNRISE
        Serial.print(" -> dim : ");
        Serial.println(dim);
#endif
        dimmer(dim);
        soft++;
      }
      else if (!doneSunrise)
      {
        doneSunrise = true;
        soft = 0;
#ifdef DEBUG_DO_AT_SUNRISE
        Serial.println("Sunrise : Dimmer = 0%");
#endif
      }
    } else doneSunrise = false;

    //////////////////////////////////////////////////////////////////
    //      _                                     _____     _
    //     | |                          _        (_____)   | |
    //      \ \  _   _ ____   ___  ____| |_         _  ___ | | _
    //       \ \| | | |  _ \ /___)/ _  )  _)       | |/ _ \| || \ 
    //   _____) ) |_| | | | |___ ( (/ /| |__    ___| | |_| | |_) )
    //  (______/ \____|_| |_(___/ \____)\___)  (____/ \___/|____/
    //
    //////////////////////////////////////////////////////////////////
    int beforeSunset = offset - soft;
    if (minutes == (sunset - beforeSunset))
    {
#ifdef DEBUG_DO_AT_SUNSET
      Serial.print(minutes);
      Serial.print(" -> Sunset : ");
      Serial.print(sunrise);
      Serial.print(" -> before sunset : ");
      Serial.println(beforeSunset);
#endif
      if (minutes <= sunset)
      {
        uint8_t d = maxLight * (1 - (float)(offset - soft) / offset);
        if (d < 5) d = 0;
        String dim = (String)d;

#ifdef DEBUG_DO_AT_SUNSET
        Serial.print(" -> dim : ");
        Serial.println(dim);
#endif
        dimmer(dim);
        soft++;
      }
      else if (!doneSunset)
      {
        doneSunset = true;
        soft = 0;
#ifdef DEBUG_DO_AT_SUNSET
        Serial.println("Sunset : Dimmer = 100%");
#endif
      }
    } else doneSunset = false;
  }

  //////////////////////////////////////////////////////////////////
  //    _   _                               _
  //   | \ | | _____      __   ___  _ __ __| | ___ _ __
  //   |  \| |/ _ \ \ /\ / /  / _ \| '__/ _` |/ _ \ '__|
  //   | |\  |  __/\ V  V /  | (_) | | | (_| |  __/ |
  //   |_| \_|\___| \_/\_/    \___/|_|  \__,_|\___|_|
  //
  //              wait for incomming data
  //////////////////////////////////////////////////////////////////
  if (xbee.available() > 0)
  {
    //End Of Frame is \n
    String messageFromXbee = xbee.readStringUntil('\n');
    /* Debug show incoming frame */
#ifdef DEBUG_SHOW_FRAME
    Serial.println(messageFromXbee);
#endif

    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //   _______                    ___                ___                          _                            _
    //  (_______)          _       / __)              / __)                        (_)      _                   (_)_
    //   _       ____  ___| |_    | |__ ___   ____   | |__ ____ ____ ____   ____    _ ____ | |_  ____ ____  ____ _| |_ _   _
    //  | |     / _  )/___)  _)   |  __) _ \ / ___)  |  __) ___) _  |    \ / _  )  | |  _ \|  _)/ _  ) _  |/ ___) |  _) | | |
    //  | |____( (/ /|___ | |__   | | | |_| | |      | | | |  ( ( | | | | ( (/ /   | | | | | |_( (/ ( ( | | |   | | |_| |_| |
    //   \______)____|___/ \___)  |_|  \___/|_|      |_| |_|   \_||_|_|_|_|\____)  |_|_| |_|\___)____)_|| |_|   |_|\___)__  |
    //                                                                                              (_____|           (____/
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    if (testChecksum(messageFromXbee))
    {
      Serial.print("Transmission OK -> it's for ");  // No error transmission

      // Test for receiver address
      String messageFor = getAddress(messageFromXbee);
      if ( messageFor == slaveAddress || messageFor == diffusionAddress) // It's for me ?
      {
        Serial.println(" slave : It's me !");  // yes
        // Prepare response message to Master
        String messageToSend = slaveAddress;
        messageToSend += SEPARATOR;

        // look for order
        String order = getOrder(messageFromXbee);
        messageToSend += order;
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //   ______                     _                              ______            _
        //  |  ___ \                   | |     /\         _           |  ___ \          | |
        //  | | _ | | ____ ____       / /     /  \  _   _| |_  ___    | | _ | | ___   _ | | ____
        //  | || || |/ _  |  _ \     / /     / /\ \| | | |  _)/ _ \   | || || |/ _ \ / || |/ _  )
        //  | || || ( ( | | | | |   / /     | |__| | |_| | |_| |_| |  | || || | |_| ( (_| ( (/ /
        //  |_||_||_|\_||_|_| |_|  |_|      |______|\____|\___)___/   |_||_||_|\___/ \____|\____)
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        if (order == "MOD")
        {
          String data = getData(messageFromXbee);
          messageToSend += SEPARATOR;
          messageToSend += data;
          if (data == "AUTO")
          {
            manualMode = false;
#ifdef DEBUG_MODE
            Serial.println("Automatic Mode active : Manual operations temporarly effective");
#endif
          }
          else if (data == "MANUAL")
          {
            manualMode = true;
#ifdef DEBUG_MODE
            Serial.println("Manual Mode active : Automatic operations disabled");
#endif
          }
          else
          {
#ifdef DEBUG_MODE
            Serial.print(data);
            Serial.println(" : Unknwon Mode");
            Serial.print("Still in mode : ");
            (manualMode) ? Serial.println("Manual") : Serial.println("Automatic");
#endif
          }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //      _                             ______                 _       _       _
        //     | |         _                 |  ___ \               | |     (_)     | |    _
        //      \ \   ____| |_ _   _ ____    | | _ | | ____ _   _   | |      _  ____| | _ | |_
        //       \ \ / _  )  _) | | |  _ \   | || || |/ _  ( \ / )  | |     | |/ _  | || \|  _)
        //   _____) | (/ /| |_| |_| | | | |  | || || ( ( | |) X (   | |_____| ( ( | | | | | |__
        //  (______/ \____)\___)____| ||_/   |_||_||_|\_||_(_/ \_)  |_______)_|\_|| |_| |_|\___)
        //                          |_|                                       (_____|
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        else if (order == "MAX")
        {
          String data = getData(messageFromXbee);
          messageToSend += SEPARATOR;
          messageToSend += data;
          maxLight = atoi(data.c_str());
#ifdef DEBUG_MAX
          Serial.print("Setup Max Light in Automatic mode is now : ");
          Serial.print(maxLight);
          Serial.println("%");
#endif
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //      _                              _____   ___  ___                        _                                       _
        //     | |         _                  / ___ \ / __)/ __)          _       _   (_)                                     (_)
        //      \ \   ____| |_ _   _ ____    | |   | | |__| |__ ___  ____| |_    | |_  _ ____   ____     ___ _   _ ____   ____ _  ___  ____
        //       \ \ / _  )  _) | | |  _ \   | |   | |  __)  __)___)/ _  )  _)   |  _)| |    \ / _  )   /___) | | |  _ \ / ___) |/___)/ _  )
        //   _____) | (/ /| |_| |_| | | | |  | |___| | |  | | |___ ( (/ /| |__   | |__| | | | ( (/ /   |___ | |_| | | | | |   | |___ ( (/ /
        //  (______/ \____)\___)____| ||_/    \_____/|_|  |_| (___/ \____)\___)   \___)_|_|_|_|\____)  (___/ \____|_| |_|_|   |_(___/ \____)
        //                          |_|
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        else if (order == "OFS")
        {
          String data = getData(messageFromXbee);
          messageToSend += SEPARATOR;
          messageToSend += data;
          offset = atoi(data.c_str());
#ifdef DEBUG_OFS
          Serial.print("Offset time before sunrise and sunset is now : ");
          Serial.print(offset);
          Serial.println(" minutes");
#endif
        }
        //
        //  Code inhibit because not enough memory
        //
        //        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //        //      _                             _                        _               _
        //        //     | |         _                 | |                      (_)_            | |
        //        //      \ \   ____| |_ _   _ ____    | |      ___  ____   ____ _| |_ _   _  _ | | ____
        //        //       \ \ / _  )  _) | | |  _ \   | |     / _ \|  _ \ / _  | |  _) | | |/ || |/ _  )
        //        //   _____) | (/ /| |_| |_| | | | |  | |____| |_| | | | ( ( | | | |_| |_| ( (_| ( (/ /
        //        //  (______/ \____)\___)____| ||_/   |_______)___/|_| |_|\_|| |_|\___)____|\____|\____)
        //        //                          |_|                         (_____|
        //        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //        else if (order == "LON")
        //        {
        //          String data = getData(messageFromXbee);
        //          LONGITUDE = data.toFloat();
        //          tardis.Position(LATITUDE, LONGITUDE);
        //#ifdef DEBUG_LATLONG
        //          Serial.print("LONGITUDE is now : ");
        //          Serial.println(LONGITUDE);
        //#endif
        //        }
        //
        //        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //        //      _                             _                 _               _
        //        //     | |         _                 | |           _   (_)_            | |
        //        //      \ \   ____| |_ _   _ ____    | |      ____| |_  _| |_ _   _  _ | | ____
        //        //       \ \ / _  )  _) | | |  _ \   | |     / _  |  _)| |  _) | | |/ || |/ _  )
        //        //   _____) | (/ /| |_| |_| | | | |  | |____( ( | | |__| | |_| |_| ( (_| ( (/ /
        //        //  (______/ \____)\___)____| ||_/   |_______)_||_|\___)_|\___)____|\____|\____)
        //        //                          |_|
        //        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //        else if (order == "LAT")
        //        {
        //          String data = getData(messageFromXbee);
        //          LATITUDE = data.toFloat();
        //          tardis.Position(LATITUDE, LONGITUDE);
        //#ifdef DEBUG_LATLONG
        //          Serial.print("LATITUDE is now : ");
        //          Serial.println(LATITUDE);
        //#endif
        //        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //      _                                _                                       _
        //     | |         _                    | |                                 _   (_)
        //      \ \   ____| |_ _   _ ____        \ \  _   _ ____  ____   ____  ____| |_  _ ____   ____
        //       \ \ / _  )  _) | | |  _ \        \ \| | | |    \|    \ / _  )/ ___)  _)| |    \ / _  )
        //   _____) | (/ /| |_| |_| | | | |   _____) ) |_| | | | | | | ( (/ /| |   | |__| | | | ( (/ /
        //  (______/ \____)\___)____| ||_/   (______/ \____|_|_|_|_|_|_|\____)_|    \___)_|_|_|_|\____)
        //                          |_|
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        else if (order == "EST")
        {
          String data = getData(messageFromXbee);
          messageToSend += SEPARATOR;
          messageToSend += data;
          if (data == "True" || data == "true" || data == "TRUE" || data == "1")
          {
            EST = true;
#ifdef DEBUG_SUMMERTIME
            Serial.println("Summer Time is enabled");
#endif
          }
          else if (data == "False" || data == "false" || data == "FALSE" || data == "0")
          {
            EST = false;
#ifdef DEBUG_SUMMERTIME
            Serial.println("Summer Time is disabled");
#endif
          }
          else
          {
#ifdef DEBUG_SUMMERTIME
            Serial.print(data);
            Serial.println(" : Unknwon EST command.");
            Serial.print("Summer Time is still  ");
            (EST) ? Serial.println("enabled.") : Serial.println("disabled.");
#endif
          }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //      _          _           _             _                         _
        //     | |        (_)_        | |       _   | |                       | |
        //      \ \  _ _ _ _| |_  ____| | _    | |_ | | _   ____     ____ ____| | ____ _   _
        //       \ \| | | | |  _)/ ___) || \   |  _)| || \ / _  )   / ___) _  ) |/ _  | | | |
        //   _____) ) | | | | |_( (___| | | |  | |__| | | ( (/ /   | |  ( (/ /| ( ( | | |_| |
        //  (______/ \____|_|\___)____)_| |_|   \___)_| |_|\____)  |_|   \____)_|\_||_|\__  |
        //                                                                            (____/
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        else if (order == "REL")
        {
          String data = getData(messageFromXbee);
          if (data == "ON")
          {
            // Switch ON the relay
            relay(ON);
          }
          else if (data == "OFF")
          {
            // Switch OFF the relay
            relay(OFF);
          }
          else
          {
            /* Debug */
            Serial.println("ERROR : Relay order unknwon !");
          }
          messageToSend += SEPARATOR;
          switch (digitalRead(RELAY_PIN))
          {
            case 0 : messageToSend += "OFF"; break;
            case 1 : messageToSend += "ON"; break;
          }

          toSend = true;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //    _____           _                     _              _____   _
        //   / ___ \         | |               _   | |            (____ \ (_)
        //  | |   | | ____ _ | | ____  ____   | |_ | | _   ____    _   \ \ _ ____  ____   ____  ____
        //  | |   | |/ ___) || |/ _  )/ ___)  |  _)| || \ / _  )  | |   | | |    \|    \ / _  )/ ___)
        //  | |___| | |  ( (_| ( (/ /| |      | |__| | | ( (/ /   | |__/ /| | | | | | | ( (/ /| |
        //   \_____/|_|   \____|\____)_|       \___)_| |_|\____)  |_____/ |_|_|_|_|_|_|_|\____)_|
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        else if (order == "DIM")
        {
          // look for data
          String data = getData(messageFromXbee);
          messageToSend += SEPARATOR;
          messageToSend += data;

          dimmer(data);
          toSend = true;
        }

        //
        else if (order == "AMP")
        {
          float current = getEffectiveCurrent();
          messageToSend += SEPARATOR;
          messageToSend += (String)current; // order courant

          toSend = true;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //     | |         _                  _   | |                   _
        //      \ \   ____| |_ _   _ ____    | |_ | | _   ____     ____| |_  ____
        //       \ \ / _  )  _) | | |  _ \   |  _)| || \ / _  )   / ___)  _)/ ___)
        //   _____) | (/ /| |_| |_| | | | |  | |__| | | ( (/ /   | |   | |_( (___
        //  (______/ \____)\___)____| ||_/    \___)_| |_|\____)  |_|    \___)____)
        //                          |_|
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        else if (order == "RTC")
        {
          String data = getData(messageFromXbee);

          tsNow = getTimestamp(data);
          setupRTC(tsNow);
          now = RTC.now();
          tsNow = now.unixtime();
          /* Debug */
#ifdef DEBUG_DATETIME
          Serial.print(tsNow);
          Serial.print(" = ");
          Serial.print(now.day());
          Serial.print("/");
          Serial.print(now.month());
          Serial.print("/");
          Serial.print(now.year());
          Serial.print(" ");
          Serial.print(now.hour());
          Serial.print(":");
          Serial.print(now.minute());
          Serial.print(":");
          Serial.println(now.second());
          if (EST)
          {
            if (summerTime(tsNow)) Serial.println("It's summer time");
            else Serial.println("It's winter time");
          }
          else Serial.println("Summer time disabled");
#endif
          toSend = true;
        }
        else
        {
          toSend = false;
          /* Debug */
          Serial.print("ERROR : ");
          Serial.print(order);
          Serial.print(" -> ");
          Serial.println("Unknown order !");
        }
        if (toSend)
        {
          messageToSend += SEPARATOR;
          messageToSend += (String)tsNow;
          messageToSend += SEPARATOR;
          messageToSend += computeChecksum(messageToSend);
          messageToSend += END_OF_FRAME;
          xbee.print(messageToSend);
        }
      }
      else Serial.println(" : Not for me !");
    }//End integrity frame test
    else Serial.println("ERROR : integrity frame !");
  }//End incomming data
}//End loop


/***********************************************************************************************
        _____                 _   _
       |  ___|   _ _ __   ___| |_(_) ___  _ __  ___   _______  _ __   ___  ___
       | |_ | | | | '_ \ / __| __| |/ _ \| '_ \/ __| |_  / _ \| '_ \ / _ \/ __|
       |  _|| |_| | | | | (__| |_| | (_) | | | \__ \  / / (_) | | | |  __/\__ \
       |_|   \__,_|_| |_|\___|\__|_|\___/|_| |_|___/ /___\___/|_| |_|\___||___/

 ************************************************************************************************/
/////////////////////////////////////////////////////////////////////////////////////////////////
//                                    _        ____ _               _
//     ___ ___  _ __ ___  _ __  _   _| |_ ___ / ___| |__   ___  ___| | _____ _   _ _ __ ___
//    / __/ _ \| '_ ` _ \| '_ \| | | | __/ _ \ |   | '_ \ / _ \/ __| |/ / __| | | | '_ ` _ \ 
//   | (_| (_) | | | | | | |_) | |_| | ||  __/ |___| | | |  __/ (__|   <\__ \ |_| | | | | | |
//    \___\___/|_| |_| |_| .__/ \__,_|\__\___|\____|_| |_|\___|\___|_|\_\___/\__,_|_| |_| |_|
// //////////////////////////////////////////////////////////////////////////////////////////////

String computeChecksum(String message)
{
  char checksum = 0;
  for (int i = 0; i < message.length(); i++)
  {
    /* Debug calcul checksum*/
#ifdef DEBUG_CHECKSUM
    Serial.print("message[");
    Serial.print(i);
    Serial.print("] = 0x");
    Serial.print(message[i], HEX);
    Serial.print(" - ");
#endif
    checksum ^= message[i];
#ifdef DEBUG_CHECKSUM
    Serial.println(checksum, HEX);
#endif
  }
  return String(checksum, HEX);
}
//////////////////////////////////////////////////////////////////////////////////////////////
//    _            _    ____ _               _
//   | |_ ___  ___| |_ / ___| |__   ___  ___| | _____ _   _ _ __ ___
//   | __/ _ \/ __| __| |   | '_ \ / _ \/ __| |/ / __| | | | '_ ` _ \ 
//   | ||  __/\__ \ |_| |___| | | |  __/ (__|   <\__ \ |_| | | | | | |
//    \__\___||___/\__|\____|_| |_|\___|\___|_|\_\___/\__,_|_| |_| |_|
//////////////////////////////////////////////////////////////////////////////////////////////
bool testChecksum(String message)
{
  //Dont take the last char (\n)
  String giveChecksum = message.substring(message.lastIndexOf(SEPARATOR) + 1, message.length());
  String frame = message.substring(0, message.lastIndexOf(SEPARATOR) + 1);
  String foundChecksum = computeChecksum(frame);
  /* Debug Test checksum */
#ifdef DEBUG_TEST_CHECKSUM
  Serial.print("Checksum on frame : ");
  Serial.print(giveChecksum);
  Serial.print(" in ");
  Serial.print(foundChecksum.length());
  Serial.println(" char");
  Serial.print("Calcul Checksum   : ");
  Serial.print(foundChecksum);
  Serial.print(" in ");
  Serial.print(foundChecksum.length());
  Serial.println(" char");
#endif

  if (foundChecksum == giveChecksum)
  {
#ifdef DEBUG_TEST_CHECKSUM
    Serial.println("OK");
#endif
    return true;
  }
  else
  {
#ifdef DEBUG_TEST_CHECKSUM
    Serial.println("NOK");
#endif
    return false;
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//    _____ _           _                                  _
//   |  ___(_)_ __   __| |  ___  ___ _ __   __ _ _ __ __ _| |_ ___  _ __
//   | |_  | | '_ \ / _` | / __|/ _ \ '_ \ / _` | '__/ _` | __/ _ \| '__|
//   |  _| | | | | | (_| | \__ \  __/ |_) | (_| | | | (_| | || (_) | |
//   |_|   |_|_| |_|\__,_| |___/\___| .__/ \__,_|_|  \__,_|\__\___/|_|
//                                  |_|
/////////////////////////////////////////////////////////////////////////////////////////////////
int firstSep(String message)
{
  return message.indexOf(SEPARATOR);
}

int secondSep(String message)
{
  return message.indexOf(SEPARATOR, firstSep(message) + 1) ;
}

int thirdSep(String message)
{
  return message.indexOf(SEPARATOR, secondSep(message) + 1) ;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//               _      _       _     _
//     __ _  ___| |_   / \   __| | __| |_ __ ___  ___ ___
//    / _` |/ _ \ __| / _ \ / _` |/ _` | '__/ _ \/ __/ __|
//   | (_| |  __/ |_ / ___ \ (_| | (_| | | |  __/\__ \__ \
//    \__, |\___|\__/_/   \_\__,_|\__,_|_|  \___||___/___/
//    |___/
////////////////////////////////////////////////////////////////////////////////////////////////
String getAddress(String message)
{

  String address = message.substring(0, firstSep(message)); // Read slave address
  Serial.print(address); // Show slave address on serial monitor
  return address;
}
/////////////////////////////////////////////////////////////////////////////////////////////////
//               _    ___          _
//     __ _  ___| |_ / _ \ _ __ __| | ___ _ __
//    / _` |/ _ \ __| | | | '__/ _` |/ _ \ '__|
//   | (_| |  __/ |_| |_| | | | (_| |  __/ |
//    \__, |\___|\__|\___/|_|  \__,_|\___|_|
//    |___/
////////////////////////////////////////////////////////////////////////////////////////////////
String getOrder(String message)
{
  return message.substring(firstSep(message) + 1, secondSep(message)); // Read order
}
/////////////////////////////////////////////////////////////////////////////////////////////////
//               _   ____        _
//     __ _  ___| |_|  _ \  __ _| |_ __ _
//    / _` |/ _ \ __| | | |/ _` | __/ _` |
//   | (_| |  __/ |_| |_| | (_| | || (_| |
//    \__, |\___|\__|____/ \__,_|\__\__,_|
//    |___/
////////////////////////////////////////////////////////////////////////////////////////////////
String getData(String message)
{
  return message.substring(secondSep(message) + 1, thirdSep(message));
}
/////////////////////////////////////////////////////////////////////////////////////////////////
//        _ _
//     __| (_)_ __ ___  _ __ ___   ___ _ __
//    / _` | | '_ ` _ \| '_ ` _ \ / _ \ '__|
//   | (_| | | | | | | | | | | | |  __/ |
//    \__,_|_|_| |_| |_|_| |_| |_|\___|_|
//
////////////////////////////////////////////////////////////////////////////////////////////////
void dimmer(String value)
{
  int pwm_pc = value.toInt();
  int pwm = map(pwm_pc, 0, 100, 255, 0);
  //int pwm = map(pwm_pc, 0, 100, 0, 255);
  analogWrite (DIMMER_PIN, pwm);
  /* Debug dimmer status */
#ifdef DEBUG_DIMMER
  Serial.print("New dimmer value : ");
  Serial.print(value);
  Serial.print("% -> pwm = ");
  Serial.println(pwm);
#endif
}
/////////////////////////////////////////////////////////////////////////////////////////////////
//             _
//    _ __ ___| | __ _ _   _
//   | '__/ _ \ |/ _` | | | |
//   | | |  __/ | (_| | |_| |
//   |_|  \___|_|\__,_|\__, |
//                     |___/
////////////////////////////////////////////////////////////////////////////////////////////////
void relay(int state)
{
  digitalWrite (RELAY_PIN, state);
  /* Debug
    Serial.print("Relay : ");
    Serial.println(state);
  */
}
/////////////////////////////////////////////////////////////////////////////////////////////////
//               _   __  __             ____                          _
//     __ _  ___| |_|  \/  | __ ___  __/ ___|   _ _ __ _ __ ___ _ __ | |_
//    / _` |/ _ \ __| |\/| |/ _` \ \/ / |  | | | | '__| '__/ _ \ '_ \| __|
//   | (_| |  __/ |_| |  | | (_| |>  <| |__| |_| | |  | | |  __/ | | | |_
//    \__, |\___|\__|_|  |_|\__,_/_/\_\\____\__,_|_|  |_|  \___|_| |_|\__|
//    |___/
////////////////////////////////////////////////////////////////////////////////////////////////
/*Function: Sample for 100ms and get the maximum value from the SIG pin*/
int getMaxCurrent()
{
  int sensorValue;             //value read from the sensor
  int sensorMax = 0;
  uint32_t start_time = millis();
  while ((millis() - start_time) < 100) //sample for 100ms
  {
    sensorValue = analogRead(CURRENT_PIN);
    if (sensorValue > sensorMax)
    {
      sensorMax = sensorValue;
    }
  }
  return sensorMax;
}
/////////////////////////////////////////////////////////////////////////////////////////////////
//               _   _____  __  __           _   _            ____                          _
//     __ _  ___| |_| ____|/ _|/ _| ___  ___| |_(_)_   _____ / ___|   _ _ __ _ __ ___ _ __ | |_
//    / _` |/ _ \ __|  _| | |_| |_ / _ \/ __| __| \ \ / / _ \ |  | | | | '__| '__/ _ \ '_ \| __|
//   | (_| |  __/ |_| |___|  _|  _|  __/ (__| |_| |\ V /  __/ |__| |_| | |  | | |  __/ | | | |_
//    \__, |\___|\__|_____|_| |_|  \___|\___|\__|_| \_/ \___|\____\__,_|_|  |_|  \___|_| |_|\__|
//    |___/
////////////////////////////////////////////////////////////////////////////////////////////////
float getEffectiveCurrent()
{
  int sensor_max = getMaxCurrent();
  float amplitude_current = (float)sensor_max / 1024 * 5 / 800 * 2000000;
  float effective_value = amplitude_current / 1.414; //minimum_current=1/1024*5/800*2000000/1.414=8.6(mA)

  /* Debug */
#ifdef DEBUG_AMPERE
  Serial.print("sensor_max = ");
  Serial.println(sensor_max);
  Serial.println("The amplitude of the current is(in mA)");
  Serial.println(amplitude_current, 1); //Only one number after the decimal point
  Serial.println("The effective value of the current is(in mA)");
  Serial.println(effective_value, 1);
#endif

  return effective_value;
}
/////////////////////////////////////////////////////////////////////////////////////////////////
//               _  _____ _                     _                        _   _
//     __ _  ___| ||_   _(_)_ __ ___   ___  ___| |_ __ _ _ __ ___  _ __ | \ | | _____      __
//    / _` |/ _ \ __|| | | | '_ ` _ \ / _ \/ __| __/ _` | '_ ` _ \| '_ \|  \| |/ _ \ \ /\ / /
//   | (_| |  __/ |_ | | | | | | | | |  __/\__ \ || (_| | | | | | | |_) | |\  | (_) \ V  V /
//    \__, |\___|\__||_| |_|_| |_| |_|\___||___/\__\__,_|_| |_| |_| .__/|_| \_|\___/ \_/\_/
//    |___/                                                       |_|
////////////////////////////////////////////////////////////////////////////////////////////////
//uint32_t getTimestampNow(DateTime now)
//{
//  return now.unixtime();
//}

/////////////////////////////////////////////////////////////////////////////////////////////////
//               _   ____        _      _____ _                _   _
//     __ _  ___| |_|  _ \  __ _| |_ __|_   _(_)_ __ ___   ___| \ | | _____      __
//    / _` |/ _ \ __| | | |/ _` | __/ _ \| | | | '_ ` _ \ / _ \  \| |/ _ \ \ /\ / /
//   | (_| |  __/ |_| |_| | (_| | ||  __/| | | | | | | | |  __/ |\  | (_) \ V  V /
//    \__, |\___|\__|____/ \__,_|\__\___||_| |_|_| |_| |_|\___|_| \_|\___/ \_/\_/
//    |___/
/////////////////////////////////////////////////////////////////////////////////////////////////
//String getDateTimeNow(DateTime now)
//{
//  uint32_t ts = now.unixtime();
//  String horodate = (String)ts + " = " + (String)now.day() + "/" + (String)now.month() +
//                    "/" + (String)now.year() + " " + (String)now.hour() +
//                    ":" + (String)now.minute() + ":" + (String)now.second();
//  return horodate;
//}

/////////////////////////////////////////////////////////////////////////////////////////////////
//               _  _____         _
//     __ _  ___| ||_   _|__   __| | __ _ _   _
//    / _` |/ _ \ __|| |/ _ \ / _` |/ _` | | | |
//   | (_| |  __/ |_ | | (_) | (_| | (_| | |_| |
//    \__, |\___|\__||_|\___/ \__,_|\__,_|\__, |
//    |___/                               |___/
////////////////////////////////////////////////////////////////////////////////////////////////
void getToday(DateTime now, byte * today)
{
  //DateTime now = RTC.now();
  today[3] = now.day();
  today[4] = now.month();
  today[5] = now.year();
}
/////////////////////////////////////////////////////////////////////////////////////////////////
//                                           _____ _
//    ___ _   _ _ __ ___  _ __ ___   ___ _ _|_   _(_)_ __ ___   ___
//   / __| | | | '_ ` _ \| '_ ` _ \ / _ \ '__|| | | | '_ ` _ \ / _ \
//   \__ \ |_| | | | | | | | | | | |  __/ |   | | | | | | | | |  __/
//   |___/\__,_|_| |_| |_|_| |_| |_|\___|_|   |_| |_|_| |_| |_|\___|
//
////////////////////////////////////////////////////////////////////////////////////////////////
bool summerTime(uint32_t t)
{
  if (EST)
  {
    DateTime myDate(t);
    DateTime winterTimeDay;
    DateTime summerTimeDay;
    uint32_t tsSummerTimeDay;
    uint32_t tsWinterTimeDay;

    DateTime march[7];
    for (int i = 0; i < 7; i++)
    {
      march[i] = DateTime(myDate.year(), 3, 25 + i, 3, 0, 0);
      if (march[i].dayOfTheWeek() == 0)
      {
        summerTimeDay = DateTime(myDate.year(), 3, march[i].day(), 3, 0, 0);
        tsSummerTimeDay = march[i].unixtime();
        break;
      }
    }

    DateTime october[7];
    for (int i = 0; i < 7; i++)
    {
      october[i] = DateTime(myDate.year(), 10, 25 + i, 3, 0, 0);
      if (october[i].dayOfTheWeek() == 0)
      {
        winterTimeDay = DateTime(myDate.year(), 10, october[i].day(), 3, 0, 0);
        tsWinterTimeDay = october[i].unixtime();
        break;
      }
    }

#ifdef DEBUG_SUMMERTIME
    //Serial.println(t);
    
    Serial.println("Compute if summer time : ");
    Serial.print("Summer Time Day : ");
    Serial.println((String)summerTimeDay.day() + "/" + (String)summerTimeDay.month() +
                   "/" + (String)summerTimeDay.year() + " " + (String)summerTimeDay.hour() +
                   ":" + (String)summerTimeDay.minute() + ":" + (String)summerTimeDay.second());
    Serial.print("Winter Time Day : ");
    Serial.println((String)winterTimeDay.day() + "/" + (String)winterTimeDay.month() +
                   "/" + (String)winterTimeDay.year() + " " + (String)winterTimeDay.hour() +
                   ":" + (String)winterTimeDay.minute() + ":" + (String)winterTimeDay.second());
#endif

    if (t > tsSummerTimeDay && t < tsWinterTimeDay) return true;
  }
  return false;
}
/////////////////////////////////////////////////////////////////////////////////////////////////
//              _     _       _       _     _
//    _ __ ___ (_) __| |_ __ (_) __ _| |__ | |_
//   | '_ ` _ \| |/ _` | '_ \| |/ _` | '_ \| __|
//   | | | | | | | (_| | | | | | (_| | | | | |_
//   |_| |_| |_|_|\__,_|_| |_|_|\__, |_| |_|\__|
//                              |___/
////////////////////////////////////////////////////////////////////////////////////////////////
bool midnight(uint32_t t)
{
  DateTime myDate(t);
  byte Hour, Minute;
  Hour  = myDate.hour();
  Minute = myDate.minute();

  if ((Hour + Minute) == 0) return true;
  return false;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//                                    _       ____                   _          ____                       _
//     ___ ___  _ __ ___  _ __  _   _| |_ ___/ ___| _   _ _ __  _ __(_)___  ___/ ___| _   _ _ __  ___  ___| |_
//    / __/ _ \| '_ ` _ \| '_ \| | | | __/ _ \___ \| | | | '_ \| '__| / __|/ _ \___ \| | | | '_ \/ __|/ _ \ __|
//   | (_| (_) | | | | | | |_) | |_| | ||  __/___) | |_| | | | | |  | \__ \  __/___) | |_| | | | \__ \  __/ |_
//    \___\___/|_| |_| |_| .__/ \__,_|\__\___|____/ \__,_|_| |_|_|  |_|___/\___|____/ \__,_|_| |_|___/\___|\__|
//                       |_|
/////////////////////////////////////////////////////////////////////////////////////////////////
void computeSunriseSunset(DateTime dt)
{
  if (summerTime(dt.unixtime()))
  {
    tardis.TimeZone(2 * 60); // TimeZone = UTC+1
    Serial.println("It's summer time");

  }
  else
  {
    tardis.TimeZone(1 * 60); // TimeZone = UTC+1
    Serial.println("It's winter time");
  }
  getToday(dt, today);
  tardis.SunRise(today);
#ifdef DEBUG_SUNRISE
  Serial.print("Sunrise: ");
  Serial.print((int) today[2]);
  Serial.print(":");
  Serial.println((int) today[1]);
#endif
  setSunrise(today[2], today[1]);
  tardis.SunSet(today);
#ifdef DEBUG_SUNSET
  Serial.print("Sunset: ");
  Serial.print((int) today[2]);
  Serial.print(":");
  Serial.println((int) today[1]);
#endif
  setSunset(today[2], today[1]);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//             _               ____ _____ ____
//    ___  ___| |_ _   _ _ __ |  _ \_   _/ ___|
//   / __|/ _ \ __| | | | '_ \| |_) || || |
//   \__ \  __/ |_| |_| | |_) |  _ < | || |___
//   |___/\___|\__|\__,_| .__/|_| \_\|_| \____|
//                      |_|
////////////////////////////////////////////////////////////////////////////////////////////////
void setupRTC(uint32_t t)
{
  DateTime myDate(t);
  DS3231 Clock;

  Clock.setClockMode(false);  // set to 24h

  Clock.setYear(myDate.year() - 2000);
  Clock.setMonth(myDate.month());
  Clock.setDate(myDate.day());
  Clock.setHour(myDate.hour());
  Clock.setMinute(myDate.minute());
  Clock.setSecond(myDate.second());
}
/////////////////////////////////////////////////////////////////////////////////////////////////
//               _  _____ _                     _
//     __ _  ___| ||_   _(_)_ __ ___   ___  ___| |_ __ _ _ __ ___  _ __
//    / _` |/ _ \ __|| | | | '_ ` _ \ / _ \/ __| __/ _` | '_ ` _ \| '_ \ 
//   | (_| |  __/ |_ | | | | | | | | |  __/\__ \ || (_| | | | | | | |_) |
//    \__, |\___|\__||_| |_|_| |_| |_|\___||___/\__\__,_|_| |_| |_| .__/
//    |___/                                                       |_|
////////////////////////////////////////////////////////////////////////////////////////////////
uint32_t getTimestamp(String t)
{
  uint32_t ts = atol(t.c_str());
  return ts;
}
/////////////////////////////////////////////////////////////////////////////////////////////////
//             _   ____                       _
//    ___  ___| |_/ ___| _   _ _ __  ___  ___| |_
//   / __|/ _ \ __\___ \| | | | '_ \/ __|/ _ \ __|
//   \__ \  __/ |_ ___) | |_| | | | \__ \  __/ |_
//   |___/\___|\__|____/ \__,_|_| |_|___/\___|\__|
//
////////////////////////////////////////////////////////////////////////////////////////////////
void setSunset(byte h, byte m)
{
  sunset = h * 60 + m;
}
/////////////////////////////////////////////////////////////////////////////////////////////////
//             _   ____                   _
//    ___  ___| |_/ ___| _   _ _ __  _ __(_)___  ___
//   / __|/ _ \ __\___ \| | | | '_ \| '__| / __|/ _ \
//   \__ \  __/ |_ ___) | |_| | | | | |  | \__ \  __/
//   |___/\___|\__|____/ \__,_|_| |_|_|  |_|___/\___|
//
////////////////////////////////////////////////////////////////////////////////////////////////
void setSunrise(byte h, byte m)
{
  sunrise = h * 60 + m;
}
/////////////////////////////////////////////////////////////////////////////////////////////////
//               _   __  __ _             _             ___        ____
//     __ _  ___| |_|  \/  (_)_ __  _   _| |_ ___  ___ / _ \ _ __ |  _ \  __ _ _   _
//    / _` |/ _ \ __| |\/| | | '_ \| | | | __/ _ \/ __| | | | '_ \| | | |/ _` | | | |
//   | (_| |  __/ |_| |  | | | | | | |_| | ||  __/\__ \ |_| | | | | |_| | (_| | |_| |
//    \__, |\___|\__|_|  |_|_|_| |_|\__,_|\__\___||___/\___/|_| |_|____/ \__,_|\__, |
//    |___/                                                                    |___/
////////////////////////////////////////////////////////////////////////////////////////////////
int getMinutesOnDay(DateTime now)
{
  //  DateTime myDate(getTimestampNow());
  byte h, m;
  h  = now.hour();
  m  = now.minute();
  return h * 60 + m;
}



